/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is the Instantbird messenging client, released
 * 2007.
 *
 * The Initial Developer of the Original Code is
 * Florian QUEZE <florian@instantbird.org>.
 * Portions created by the Initial Developer are Copyright (C) 2007
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

richlistitem {
  -moz-binding: url("chrome://messenger/content/chat/imAccount.xml#account");
}

richlistitem .account-buttons {
  -moz-binding: url("chrome://messenger/content/chat/imAccount.xml#nobuttons");
}
richlistitem[selected="true"] .account-buttons {
  -moz-binding: url("chrome://messenger/content/chat/imAccount.xml#buttons");
}

richlistitem:not([state="connected"]) .connected,
richlistitem:not([state="connecting"]) .connecting,
richlistitem:not([state="disconnected"]) .disconnected,
richlistitem:not([state="disconnecting"]) .disconnecting,
richlistitem:not([error="true"]) .error,
richlistitem:not([error="true"]) .errorIcon,
richlistitem:not([state="disconnected"]) .error,
richlistitem[error="true"] .disconnected,
richlistitem[selected="true"]:not([state="disconnected"]) .connectButton,
richlistitem[selected="true"][state="disconnected"] .disconnectButton,
richlistitem[selected="true"][state="disconnecting"] .disconnectButton,
richlistitem:not([selected="true"]) .autoSignOn,
richlistitem:not([reconnectPending="true"]) description[anonid="reconnect"]
{
  display: none;
}

#statusTypeIcon {
  cursor: pointer;
}

#displayName,
#statusMessage:not([statusType="offline"]) {
  cursor: text;
}

#displayName[editing],
#statusMessage[editing] {
  -moz-appearance: textfield;
  -moz-binding: url('chrome://global/content/bindings/textbox.xml#textbox');
}
