/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is the Instantbird messenging client, released
 * 2009.
 *
 * The Initial Developer of the Original Code is
 * Florian QUEZE <florian@instantbird.org>.
 * Portions created by the Initial Developer are Copyright (C) 2009
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

/* Mostly copied from the download manager */

/* List Items */
richlistitem {
  padding-top: 6px;
  padding-bottom: 6px;
  -moz-padding-start: 4px;
  -moz-padding-end: 4px;
  min-height: 25px;
%ifndef XP_MACOSX
%ifdef XP_WIN
  border-bottom: 1px solid ThreeDLightShadow;
%else
  border-bottom: 1px dotted #C0C0C0;
%endif
%endif
}

%ifdef XP_MACOSX
richlistitem:not([selected="true"]):nth-child(odd) {
  background-color: -moz-oddtreerow;
}
%endif

%ifdef XP_WIN
richlistitem[selected="true"] {
  background-image: url(chrome://mozapps/skin/extensions/itemEnabledFader.png);
}

richlistitem[selected="true"]:not(:-moz-window-inactive) {
  background-color: Highlight;
  color: HighlightText;
}
%else
%ifdef XP_MACOSX
richlistitem[selected="true"]:not(:-moz-window-inactive) {
  background-color: Highlight;
  color: HighlightText;
}
%else
richlistitem[selected="true"] {
  background-color: -moz-Dialog;
  color: -moz-DialogText;
}
%endif
%endif
