pref("messenger.options.messagesStyle.theme", "messenger-messagestyles");
pref("messenger.options.emoticonsTheme", "messenger-emoticons");
pref("messenger.conversations.textbox.autoResize", true);
pref("messenger.conversations.doubleClickToReply", true);
