/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is mozilla.org code.
 *
 * The Initial Developer of the Original Code is
 * Netscape Communications Corporation.
 * Portions created by the Initial Developer are Copyright (C) 1998
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *   Joe Hewitt <hewitt@netscape.com>
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either of the GNU General Public License Version 2 or later (the "GPL"),
 * or the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

/* ==== communicator.css ====================================================
   == Styles shared everywhere throughout the Communicator suite.
   ========================================================================== */

@import url("chrome://global/skin/");
@import url("chrome://communicator/content/communicator.css");
@import url("chrome://communicator/skin/brand.css");
@import url("chrome://communicator/skin/button.css");
@import url("chrome://communicator/skin/toolbar.css");

@namespace url("http://www.mozilla.org/keymaster/gatekeeper/there.is.only.xul");

/* ::::: online/offline icons ::::: */

#offline-status[offline="true"] {
  list-style-image: url("chrome://communicator/skin/icons/offline.gif");
}

#offline-status {
  list-style-image: url("chrome://communicator/skin/icons/online.gif");
}

/* ::::: spell checker ::::: */

.spell-suggestion {
  font-weight: bold;
}

/* ::::: directional button icons ::::: */

.up {
  min-width: 0px;
  list-style-image: url("chrome://global/skin/arrow/arrow-up.gif");
}

.up[disabled="true"] {
  list-style-image: url("chrome://global/skin/arrow/arrow-up-dis.gif");
}

.down {
  min-width: 0px;
  list-style-image: url("chrome://global/skin/arrow/arrow-dn.gif");
}

.down[disabled="true"] {
  list-style-image: url("chrome://global/skin/arrow/arrow-dn-dis.gif");
}

.sidebarTree {
  border: none;
  margin: 0px !important;
}

/* ::::: toolbar print button ::::: */
#print-button {
  list-style-image: url("chrome://communicator/skin/icons/common.png");
  -moz-image-region: rect(0 42px 39px 0);
}

#print-button:hover {
  -moz-image-region: rect(0 84px 39px 42px);
}

#print-button:hover:active {
  -moz-image-region: rect(0 126px 39px 84px);
}

#print-button[disabled="true"] {
  -moz-image-region: rect(0 168px 39px 126px) !important;
}

toolbar[iconsize="small"] > toolbarpaletteitem > #print-button,
toolbar[iconsize="small"] > #print-button {
  list-style-image: url("chrome://communicator/skin/icons/common-small.png");
  -moz-image-region: rect(0 19px 19px 0);
}

toolbar[iconsize="small"] > #print-button:hover {
  -moz-image-region: rect(0 38px 19px 19px);
}

toolbar[iconsize="small"] > #print-button:hover:active {
  -moz-image-region: rect(0 57px 19px 38px);
}

toolbar[iconsize="small"] > #print-button[disabled="true"] {
  -moz-image-region: rect(0 76px 19px 57px) !important;
}
