/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is Mozilla Thunderbird.
 *
 * The Initial Developer of the Original Code is
 * Florian QUEZE <florian@queze.net>.
 * Portions created by the Initial Developer are Copyright (C) 2011
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

imconversation {
  -moz-binding: url("chrome://messenger/content/chat/imconversation.xml#conversation");
}

imgroup {
  -moz-binding: url("chrome://messenger/content/chat/imgroup.xml#group");
  -moz-box-align: center;
}

imcontact {
  -moz-binding: url("chrome://messenger/content/chat/imcontact.xml#contact");
  -moz-box-align: center;
}

.startChatBubble, .closeConversationButton {
  cursor: pointer;
  -moz-user-focus: ignore;
  display: none;
}

imcontact[cansend]:hover .startChatBubble,
imconv:hover .closeConversationButton {
  display: -moz-box;
}

imconv {
  -moz-binding: url("chrome://messenger/content/chat/imconv.xml#conv");
  -moz-box-align: center;
}

.convUnreadCount[value="0"],
.convUnreadTargetedCount[value="0"] {
  display: none;
}

tooltip[type="buddy"] {
  -moz-binding: url("chrome://messenger/content/chat/imbuddytooltip.xml#tooltip");
}

browser[type="content-conversation"] {
  -moz-binding: url("chrome://chat/content/convbrowser.xml#browser");
}

#contextPane:not([chat]) > .conv-chat {
  display: none;
}

.conv-top-info {
  -moz-binding: url("chrome://messenger/content/chat/imconversation.xml#conv-info-large") !important;
}

#IMSearchInput {
  -moz-binding: url("chrome://messenger/content/chat/imsearch.xml#IMGlodaSearch");
}


#statusTypeIcon {
  cursor: pointer;
}

#statusMessage {
  width: 15em;
}

#statusMessage:not([statusType="offline"]) {
  cursor: text;
}

#statusMessage[editing] {
  -moz-appearance: textfield;
  -moz-binding: url('chrome://global/content/bindings/textbox.xml#textbox');
}
