var x = -false;
switch(x) {
    case 11: 
    let y = 42;
}
switch(x) {
    case 11: 
    let y = 42;
    let z = 'ponies';
}
switch(x) {
    case 11: 
    let y = 42;
    let z = 'ponies';
    let a = false;
}
