/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is mozilla.org code.
 *
 * The Initial Developer of the Original Code is
 * the Mozilla Organization.
 * Portions created by the Initial Developer are Copyright (C) 1998-2002
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *   Robert Kaiser <KaiRo@KaiRo.at>
 *   Karsten Düsterloh <mnyromyr@tprac.de>
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

/* ===== start.css =====================================================
   == Styles used by the mailnews start page.
   ======================================================================= */

body {
  margin: 0;
  padding: 0;
  background: url("chrome://branding/content/messenger-start-bg.png") no-repeat fixed center 3em;
  background-size: auto 85%;
}

h1 {
  background: url("chrome://branding/content/messenger-start-hdr.png") repeat-x bottom;
  border-bottom: 2px solid #26269A;
  font-size: 1.5em;
  padding: 0.25em 0.67em; /* 0.67*1.5=1, so fit with 1em below */
  margin: 0;
}

#main {
  margin: 0 1em;
}

p {
  margin: 0.5em 0;
}

h2 {
  font-size: 1.2em;
  font-weight: bold;
}

ul {
  margin: 0.5em 0;
}

